# encoding: utf-8
from django.utils.safestring import mark_safe
from rest_framework.utils.formatting import _remove_leading_indent, markup_description
from django.conf import settings


def get_filter_fields(view):
    filter_fields = list(getattr(view, 'filter_fields', []))
    for filter_backend in getattr(view, 'filter_backends', []):
        if hasattr(filter_backend, 'filter_field'):
            filter_fields.append(filter_backend.filter_field)
    return filter_fields


def get_view_doc(view, html=True):
    """
    Build view documentation. Return in html format.
    If you want in markdown format, use html=False
    """
    description = view.__doc__ or ''
    description = _remove_leading_indent(description)

    # include filters in description
    filter_fields = get_filter_fields(view)
    if filter_fields:
        filter_doc = ['\n\n\n## Filters', '']
        for f in filter_fields:
            filter_doc.append('- `%s`' % f)
        description += '\n'.join(filter_doc)

    # replace {api_url} by current base url
    api_url = "%s/api" % settings.HOME_URL
    description = description.replace('{api_url}', api_url)
    if html:
        description = markup_description(description)
    return description


def wrap_accordion(text_body_list):
    """
    Wrap text_body_list in twitter bootstrap accordion. text_body_list must be list with tuple with title and body
    """
    html = ['<div class="accordion" id="accordion2">']

    for i, item in enumerate(text_body_list):
        params = {
            'index': i,
            'title': item[0],
            'body': item[1]
        }

        html.append('''
<div class="accordion-group">
    <div class="accordion-heading">
      <a class="accordion-toggle" data-toggle="collapse" data-parent="#accordion2" href="#collapse%(index)d">
        %(title)s
      </a>
    </div>
    <div id="collapse%(index)d" class="accordion-body collapse">
      <div class="accordion-inner">
        %(body)s
      </div>
    </div>
  </div>
  <br/>
''' % params)
    return mark_safe('\n'.join(html))



# encoding: utf-8
import logging
from functools import wraps
from django.core.exceptions import ValidationError, PermissionDenied

LOG = logging.getLogger(__name__)
LOG_PERM = logging.getLogger('%s.perm' % __name__)

METHOD_PERMISSION_TRANSLATION = {
    'create': 'add',
    'update': 'change'
}


def build_permission_name(model_class, prefix):
    """ Build permission name for model_class (like 'app.add_model'). """
    model_name = model_class._meta.object_name.lower()
    app_label = model_class._meta.app_label
    action_name = prefix
    perm = '%s.%s_%s' % (app_label, action_name, model_name)
    return perm


class checkpermission(object):

    """
    Decorator only to BaseService methods, to protect it from unauthorized calls.
    If no arguments given, permission will be build from method name.
    For example, for method 'start' in VirtualMachineService
    the default permission required will be 'virtualmachine.start_virtualmachine'
    where the first virtualmachine is appname (in this situation same as class name)

    If you like, you can customize permission, only with prefix. For method called
    'start', you can specify:
    @checkpermission(prefix="iniciar")
    to use permission 'virtualmachine.iniciar_virtualmachine'

    Or if you want total control, you can specify entire permission as:
    @checkpermission(permission="virtualmachine.myops_vm")
    """

    def __init__(self, prefix=None, permission=None):
        self.prefix = prefix
        self.permission = permission

    def __call__(self, func):

        @wraps(func)
        def __check__(service, *args, **kwargs):
            perm_name = self.get_permission(service, func)
            obj = args[0] if len(args) else None
            call_name = "%s.%s" % (type(service).__name__, func.__name__)
            self.has_perm(service, perm_name, obj, call_name)
            return func(service, *args, **kwargs)
        __check__.checkpermission = self
        return __check__

    def get_permission(self, service, func):
        """
        Build permission required to access function "func"
        """
        if self.permission:
            perm = self.permission

        elif self.prefix is False:
            # No permission will be checked
            perm = False

        elif self.prefix:
            perm = build_permission_name(service.model_class, self.prefix)

        else:
            name = func.__name__
            # check if there is a translation between default permission and method name
            action_name = METHOD_PERMISSION_TRANSLATION.get(name, name)

            perm = build_permission_name(service.model_class, action_name)
        return perm

    def has_perm(self, service, perm_name, obj, call_name):
        """
        Raise PermissionDenied if user has no permission in object
        """
        user = service.user
        if not (perm_name is False):
            if not user.has_perm(perm_name, obj=obj):
                LOG_PERM.warn(
                    u'User %s has no permission %s. Access to %s with obj=%s',
                    user, perm_name, call_name, obj)
                raise PermissionDenied(u'User %s has no permission %s for object %s' % (service.user, perm_name, obj))

            LOG_PERM.debug(
                u'User %s was authorized to access %s with permission %s with obj=%s',
                user, call_name, perm_name, obj)


class nocheckpermission(checkpermission):

    """
    Disable checkpermission for a method
    """

    def __init__(self):
        super(nocheckpermission, self).__init__(False)


class CheckMethodPermissions(type):

    def __new__(cls, classname, bases, classdict):

        no_perm = ('model_class',)

        for attr, item in classdict.items():
            if not (attr in no_perm) and not attr.startswith('_') and callable(item):
                if not hasattr(item, 'checkpermission'):
                    # comment line bellow if you want disable checkpermission
                    classdict[attr] = checkpermission()(item)

        return type.__new__(cls, classname, bases, classdict)


class BaseService(object):
    __metaclass__ = CheckMethodPermissions
    # no_perm = ('model_class',)

    '''
    Base service class. All bussiness logic must be in services class.
    Methods, not starting with '_', will have check permission before call it.
    If you want to change permission required to execute that method, you can
    put an attribute "perm" in method. If perm is string, it is own permission name.
    If perm is method (or lambda), it will executed before and need to raise PermissionError
    if no permission
    '''

    def __init__(self, request=None, user=None):
        assert request or user, 'Both request and user are None'
        self.request = request
        self.user = user or request.user
        assert self.model_class is not None, 'model_class'

    @nocheckpermission()
    def validate(self, obj):
        """ Raises django.core.exceptions.ValidationError if any validation error exists """

        if not isinstance(obj, self.model_class):
            raise ValidationError('Invalid object(%s) for service %s' % (type(obj), type(self)))
        LOG.debug(u'Object %s state: %s', self.model_class, obj.__dict__)
        obj.full_clean()

    @nocheckpermission()
    def filter_objects(self, objects, perm=None):
        """ Return only objects with specified permission in objects list. If perm not specified, 'view' perm will be used. """
        if perm is None:
            perm = build_permission_name(self.model_class, 'view')
        return filter(lambda o: self.user.has_perm(perm, obj=o), objects)

    def __repr__(self):
        return '%s(user=%s)' % (type(self).__name__, self.user)

    def __str__(self):
        return repr(self)



# encoding: utf-8
import logging
import numbers
from .core import BaseService, CheckMethodPermissions, build_permission_name, \
    nocheckpermission
from django.core.exceptions import PermissionDenied

LOG = logging.getLogger(__name__)
LOG_PERM = logging.getLogger('%s.perm' % __name__)


class CreateServiceMixin(object):

    ''' Add default create operation to service '''
    __metaclass__ = CheckMethodPermissions

    def create(self, obj):
        LOG.info(u'Creating %s - %s', type(obj).__name__, obj)
        self.validate(obj)
        obj.save()


class UpdateServiceMixin(object):

    ''' Add default update operation to service '''
    __metaclass__ = CheckMethodPermissions

    def update(self, obj):
        LOG.info(u'Updating %s - %s', type(obj).__name__, obj)
        obj.save()


class DeleteServiceMixin(object):

    ''' Add default delete operation to service '''
    __metaclass__ = CheckMethodPermissions

    def delete(self, obj):
        LOG.info(u'Deleting %s - %s', type(obj).__name__, obj)
        obj.delete()


class ListServiceMixin(object):

    """
    Performe pre-filter in object list to avoid unauthorized access
    """

    @nocheckpermission()
    def list(self, **filters):
        """ Returns a queryset filtering object by user permission. If you want,
        you can specify filter arguments.

        See https://docs.djangoproject.com/en/dev/ref/models/querysets/#filter for more details
        """
        LOG.debug(u'Querying %s by filters=%s', self.model_class.__name__, filters)
        query = self.__queryset__()
        perm = build_permission_name(self.model_class, 'view')
        query_with_permission = filter(lambda o: self.user.has_perm(perm, obj=o), query)
        ids = map(lambda o: o.pk, query_with_permission)

        # FIXME: Return to query again without use database
        queryset = self.__queryset__().filter(pk__in=ids)
        related = getattr(self, 'select_related', None)
        if related:
            queryset = queryset.select_related(*related)

        return queryset

    def __queryset__(self):
        """ Returns basic queryset """
        return self.model_class.objects.get_query_set()

    @nocheckpermission()
    def get(self, pk=None, **filters):
        """ Retrieve an object instance. If a single argument is supplied, object is queried by
        primary key, else filter queries will be applyed.
        If more than one object was found raise MultipleObjectsReturned.
        If no object found, raise DoesNotExist.
        Raise PermissionDenied if user has no permission 'view' on object.

        See https://docs.djangoproject.com/en/dev/ref/models/querysets/#get for more details
        """
        LOG.debug(u'Querying (GET) %s by pk=%s and filters=%s', self.model_class.__name__, repr(pk), filters)
        query = self.model_class.objects.filter(**filters)
        if pk is None:
            obj = query.get()
        else:
            if (isinstance(pk, basestring) and pk.isdigit()) or isinstance(pk, numbers.Number):
                obj = query.get(pk=pk)
            elif 'slug' in self.model_class._meta.get_all_field_names():
                obj = query.get(slug=pk)
            else:
                # pk is not a number and model has no slug. So, object don't exists.
                raise self.model_class.DoesNotExist()

        perm = build_permission_name(self.model_class, 'view')
        if not self.user.has_perm(perm, obj=obj):
            raise PermissionDenied(u'User %s has no permission %s for object %s' % (self.user, perm, obj))
        return obj


class CRUDService(CreateServiceMixin, UpdateServiceMixin, DeleteServiceMixin, ListServiceMixin, BaseService):
    pass



"""
Force import of all modules in this package in order to get the standard test
runner to pick up the tests.  Yowzers.
"""
from __future__ import unicode_literals, print_function
import os
import traceback
import django

modules = [filename.rsplit('.', 1)[0]
           for filename in os.listdir(os.path.dirname(__file__))
           if filename.endswith('.py') and not filename.startswith('_')]
__test__ = dict()

if django.VERSION < (1, 6):
    for module in modules:
        print('loading module %-20s' % module, end='')
        try:
            exec("from django_services.tests.%s import *" % module)
            print('ok')
        except:
            print('error\n%s' % traceback.format_exc())
            raise





# encoding: utf-8
import re
import logging
from rest_framework import authentication
from rest_framework import exceptions
from plugins.keystone.service import KeystoneService

LOG = logging.getLogger(__name__)
AWS_AUTHORIZATION_HEADER_FORMAT = re.compile(r'\AAWS ([^:]+):(.*)\Z')


class KeystoneAuthentication(authentication.BaseAuthentication):

    def authenticate(self, request):
        match = AWS_AUTHORIZATION_HEADER_FORMAT.match(request.META.get('HTTP_AUTHORIZATION', ''))
        if match:
            # validate required headers
            for header in ['HTTP_DATE', 'CONTENT_TYPE']:
                if header not in request.META:
                    LOG.info(u'invalid aws authentication. No header %s' % header)
                    raise exceptions.AuthenticationFailed('No Header %s' % header)

            access_key = match.group(1)
            signature = match.group(2)
            verb = request.method
            path = request.get_full_path()
            content = request.body

            content_type = request.META['CONTENT_TYPE']
            date = request.META['HTTP_DATE']
            user, tenant = self.validate_rest_request(access_key,
                                                      signature,
                                                      verb,
                                                      path,
                                                      date,
                                                      content,
                                                      content_type)
            if not user or not tenant:
                LOG.info(u'Invalid credentiais')
                raise exceptions.AuthenticationFailed('No AWS credentiais')

            # put user in request
            # Do not use django.contrib.auth.login method, because I don't want a new session on every api call
            request._request.user = user
            return user, None

        # This request probably will be authenticated by another provider (or is unauthenticated)
        LOG.debug(u'No header http authentication or not in format "AWS xxx:xxx"')
        return None

    def keystone(self):
        return KeystoneService()

    def validate_rest_request(self, access_key, signature, verb, path, date, content, content_type):
        user, tenant = self.keystone().validate_rest_request(access_key,
                                                             signature,
                                                             verb,
                                                             path,
                                                             date,
                                                             content,
                                                             content_type)
        return user, tenant



# encoding: utf-8
import logging
import json
from functools import wraps
from django.http import Http404
from rest_framework import generics, mixins, status, viewsets
from rest_framework.response import Response
from application.core.exceptions import InvalidOperationException

LIST = 'list'
CREATE = 'create'
RETRIEVE = 'retrieve'
UPDATE = 'update'
DESTROY = 'destroy'
DEFAULT_OPERATIONS = frozenset([LIST, RETRIEVE])
ALL_OPERATIONS = frozenset([LIST, CREATE, RETRIEVE, UPDATE, DESTROY])

LOG = logging.getLogger(__name__)


def make_json(detail):
    return json.dumps({'detail': detail})


def exception_translation(func):
    """
    Catch exception and build correct api response for it.
    """
    @wraps(func)
    def decorator(*arg, **kwargs):
        try:
            return func(*arg, **kwargs)
        except InvalidOperationException, e:
            return Response(status=status.HTTP_412_PRECONDITION_FAILED, data={'detail': e.message}, headers={'Content-Type': 'application/json'})
    return decorator


def getattr_in_cls_list(cls_list, attr, default):
    """ Search for an attribute (attr) in class list (cls_list). Returns
    attribute value if exists or None if not. """
    for cls in cls_list:
        if hasattr(cls, attr):
            return getattr(cls, attr)
    return default


class create_api_class(type):
    def __new__(mcs, name, bases, attrs):

        if name == 'OrquestraAPI':
            # use default class creator
            return type.__new__(mcs, name, bases, attrs)

        new_bases = list(bases)

        # remove object class
        if object in new_bases:
            new_bases.remove(object)

        # extract operations attribute. Try to search subclasses
        if 'operations' in attrs:
            operations = attrs['operations']
        else:
            operations = getattr_in_cls_list(new_bases, 'operations', DEFAULT_OPERATIONS)

        # verify if all operations are known.
        if not ALL_OPERATIONS.issuperset(operations):
            raise RuntimeError("Invalid operations: %s in %s" % (list(set(operations) - ALL_OPERATIONS), name))

        if LIST in operations:
            new_bases.append(mixins.ListModelMixin)

        if CREATE in operations:
            new_bases.append(CreateModelUsingService)

        if RETRIEVE in operations:
            new_bases.append(mixins.RetrieveModelMixin)

        if UPDATE in operations:
            new_bases.append(UpdateModelUsingService)

        if DESTROY in operations:
            new_bases.append(DestroyModelUsingService)

        new_bases += (generics.SingleObjectAPIView, generics.MultipleObjectAPIView,)

        return type.__new__(mcs, name, tuple(new_bases), attrs)


class DestroyModelUsingService(mixins.DestroyModelMixin):
    """
    Use service model to call destroy
    """
    @exception_translation
    def destroy(self, request, *args, **kwargs):
        obj = self.get_object()
        self.service.delete(obj)
        return Response(status=status.HTTP_204_NO_CONTENT)


class CreateModelUsingService(mixins.CreateModelMixin):
    """
    Use service model to call create
    """

    @exception_translation
    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.DATA, files=request.FILES)

        if serializer.is_valid():
            self.pre_save(serializer.object)
            self.object = self.service.create(serializer.object)
            self.post_save(self.object, created=True)
            headers = self.get_success_headers(serializer.data)
            return Response(serializer.data, status=status.HTTP_201_CREATED,
                            headers=headers)

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class UpdateModelUsingService(mixins.UpdateModelMixin):
    """
    Use service model to call update
    """
    @exception_translation
    def update(self, request, *args, **kwargs):
        partial = kwargs.pop('partial', False)
        self.object = self.get_object()
        serializer = self.get_serializer(self.object, data=request.DATA,
                                         files=request.FILES, partial=partial)

        if serializer.is_valid():
            self.pre_save(serializer.object)
            self.object = self.service.update(self.object)
            self.post_save(self.object, created=False)
            return Response(serializer.data, status=status.HTTP_200_OK)

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class OrquestraAPI(viewsets.GenericViewSet):
    __metaclass__ = create_api_class

    @property
    def http_request(self):
        return self.request._request

    @property
    def user(self):
        return self.http_request and self.http_request.user

    @property
    def service(self):
        '''
        Instantiate service class with django http_request
        '''
        service_class = getattr(self, 'service_class', None)
        if service_class:
            service = service_class(self.http_request)
        else:
            service = None
        return service

    def get_queryset(self):
        # allow serializer without service
        if self.service:
            query = self.service.list()
        else:
            query = super(OrquestraAPI, self).get_queryset()
        return query

    def get_object(self, queryset=None):
        """
        Override default to add support for object-level permissions.
        """
        try:
            pk = self.kwargs.get('pk', None)

            # allow serializer without service
            if self.service:
                obj = self.service.get(pk)
            else:
                obj = super(OrquestraAPI, self).get_object(queryset)
            return obj
        except self.model.DoesNotExist:
            raise Http404()

    def response_object(self, obj, status=status.HTTP_200_OK):
        serializer = self.get_serializer(instance=obj)
        return Response(serializer.data, status=status)



# encoding: utf-8
import logging
from rest_framework.routers import SimpleRouter
from .serializers import OrquestraSerializer
from .api import OrquestraAPI, exception_translation

router = SimpleRouter()
LOG = logging.getLogger(__name__)

__all__ = ['OrquestraSerializer', 'OrquestraAPI', 'exception_translation', 'register']


def register(name, viewsets, *args):
    router.register(name, viewsets, *args)
    LOG.debug(u'Registered api %s with viewset %s', name, viewsets)



# encoding: utf-8
import logging
import os.path
from importlib import import_module
from django.conf import settings
from django.conf.urls import patterns, url, include
from django.views.generic.simple import direct_to_template
from application.api import router

LOG = logging.getLogger(__name__)


for app in settings.INSTALLED_APPS:
    if not app.startswith('application.'):
        continue

    try:
        base_module = import_module(app)
        api = import_module("%s.api" % app)
    except ImportError, e:
        if os.path.exists(os.path.abspath(os.path.join(base_module.__file__, '../api.py'))):
            # File exists, error on import
            raise

urlpatterns = patterns(
    'application.api.views',
    url(r'^$', 'api_help', name='api.index'),
    url(r'^', include(router.urls)),
    url(r'^api-auth/', include('rest_framework.urls', namespace='rest_framework'))
)



# encoding: utf-8
import logging
from rest_framework import renderers
from django.conf import settings
from .utils import wrap_accordion, get_view_doc

LOG = logging.getLogger(__name__)


class CustomBrowsableAPIRenderer(renderers.BrowsableAPIRenderer):

    def get_description(self, view):

        view_doc = get_view_doc(view)
        # convert to html
        description = wrap_accordion([('Documentation', view_doc)])
        return description

    def show_form_for_method(self, view, method, request, obj):
        if not getattr(settings, 'ALLOW_API_BROWSER_USE_CHANGING_METHODS', True) and not (method in ['GET', 'OPTIONS']):
            LOG.debug("Browser render don't allow changing methods")
            return False

        return super(CustomBrowsableAPIRenderer, self).show_form_for_method(view, method, request, obj)



# encoding: utf-8
from mock import patch, MagicMock
from django.test import TestCase
from django.contrib.auth.models import User
from django.core.exceptions import ValidationError
from django.http import HttpRequest
from django_services import service as svc


class TestModel(MagicMock):

    class _meta:
        object_name = "TestModel"
        app_label = "test"


class CRUDServiceForTest(svc.CRUDService):
    model_class = TestModel

    def my_operation(self, obj):
        pass


class ServiceForTest(svc.BaseService):
    model_class = TestModel


class BaseServiceTest(TestCase):

    def test_after_instantiate_put_user_as_instance_attribute(self):
        user = User()
        service = ServiceForTest(user=user)
        self.assertIs(user, service.user)
        self.assertIsNone(service.request)

    def test_after_instantiate_put_request_as_instance_attribute(self):
        request = HttpRequest()
        request.user = User()
        service = ServiceForTest(request)
        self.assertIs(request, service.request)
        self.assertIs(request.user, service.user)


class CRUDServiceTest(TestCase):

    def create_service(self):
        request = HttpRequest()
        request.user = User()
        return CRUDServiceForTest(request)

    def create_model(self):
        obj = TestModel()
        return obj

    @patch.object(User, 'has_perm')
    def test_create_will_call_obj_save_method(self, has_perm):
        service = self.create_service()
        obj = self.create_model()
        has_perm.return_value = True
        service.create(obj)
        self.assertTrue(obj.save.called)

    def test_validate_will_call_obj_full_clean_method(self):
        service = self.create_service()
        obj = self.create_model()
        service.validate(obj)
        self.assertTrue(obj.full_clean.called)

    @patch.object(User, 'has_perm')
    @patch.object(CRUDServiceForTest, 'validate')
    def test_before_save_call_validate_model(self, validate, has_perm):
        service = self.create_service()
        obj = self.create_model()
        validate.return_value = None
        has_perm.return_value = True
        service.create(obj)
        self.assertTrue(validate.called)

    def test_validate_with_different_object_type_raise_exception(self):
        service = self.create_service()
        obj = object()
        with self.assertRaises(ValidationError):
            service.validate(obj)

    @patch.object(User, 'has_perm')
    def test_create_method_will_check_add_permission(self, has_perm):
        service = self.create_service()
        obj = self.create_model()
        has_perm.return_value = True
        service.create(obj)
        has_perm.assert_called_with('test.add_testmodel', obj=obj)

    @patch.object(User, 'has_perm')
    def test_update_method_will_check_change_permission(self, has_perm):
        service = self.create_service()
        obj = self.create_model()
        has_perm.return_value = True
        service.update(obj)
        has_perm.assert_called_with('test.change_testmodel', obj=obj)

    @patch.object(User, 'has_perm')
    def test_create_method_will_validate_delete_permission(self, has_perm):
        service = self.create_service()
        obj = self.create_model()
        has_perm.return_value = True
        service.delete(obj)
        has_perm.assert_called_with('test.delete_testmodel', obj=obj)

    @patch.object(User, 'has_perm')
    def test_my_custom_method_will_check_permission_with_method_name(self, has_perm):
        service = self.create_service()
        obj = self.create_model()
        has_perm.return_value = True
        service.my_operation(obj)
        has_perm.assert_called_with('test.my_operation_testmodel', obj=obj)



import os
import sys
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

STATUS_PROD = 'Development Status :: 5 - Production/Stable'
STATUS_BETA = 'Development Status :: 4 - Beta'
STATUS_ALPHA = 'Development Status :: 3 - Alpha'

version = '0.0.1'
README = os.path.join(os.path.dirname(__file__), 'README.rst')
long_description = open(README).read()


class Tox(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True
    def run_tests(self):
        #import here, cause outside the eggs aren't loaded
        import tox
        errno = tox.cmdline(self.test_args)
        sys.exit(errno)


setup(
    name='django-services',
    version=version,
    description="Django Services Pattern Support",
    long_description=long_description,
    classifiers=[
        STATUS_PROD,
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities'],
    keywords='django model admin rest api service',
    author='Silvano Buback',
    author_email='snbuback@gmail.com',
    url='https://github.com/snbuback/django-services',
    license='APACHE2',
    packages=find_packages('.', exclude=('testproject*',)),
    include_package_data=True,
    require=['Django', 'mock'],
    tests_require=['tox'],
    cmdclass = {'test': Tox},
)



from django.test import TestCase, Client
from .auth import KeystoneAuthentication
from rest_framework.views import APIView
from django.http import HttpResponse
from rest_framework.compat import patterns
from rest_framework import status
from email.Utils import formatdate
import hmac
import base64
import hashlib


class MockView(APIView):

    def get(self, request):
        return HttpResponse({'a': 1, 'b': 2, 'c': 3})
    post = get
    put = get


urlpatterns = patterns(
    '',
    (r'^keystone-test/$', MockView.as_view(authentication_classes=[KeystoneAuthentication])),
)


class KeystoneAuthenticationIntegrationTests(TestCase):
    urls = __name__

    def setUp(self):
        self.client = Client()
        self.access_key = '3115891eb7254482a30275c7b7bf132d'
        self.secret_key = '5426e3666c6b40a89e159ce6df36d327'
        self.server_string = 'http://localhost:8001'

    def hmac_sha1(self, string_to_sign):
        m = hmac.new(self.secret_key, digestmod=hashlib.sha1)
        m.update(string_to_sign.encode('utf-8'))
        signature = base64.b64encode(m.digest())
        return signature

    def authorization_header(self, method, path, content, content_type, date=None):
        date = date or formatdate()
        string_to_sign = "\n".join((method, content, content_type, date, path))
        signature = self.hmac_sha1(string_to_sign)
        return 'AWS %s:%s' % (self.access_key, signature)

    def test_request_without_authorization_headers_returns_unauthorized(self):
        response = self.client.get('/keystone-test/')
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_request_with_authorization_headers_without_aws_returns_unauthorized(self):
        response = self.client.get('/keystone-test/', HTTP_AUTHORIZATION='BANANA')
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_request_without_date_header_returns_unauthorized(self):
        response = self.client.get('/keystone-test/', HTTP_AUTHORIZATION='AWS dd:ddd')
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_request_signed_returns_authorized(self):
        from plugins.keystone.service import KeystoneService

        def validate_rest_request(*args, **kwargs):
            from django.contrib.auth.models import User
            return User(), 'admin'
        KeystoneService.validate_rest_request = validate_rest_request

        date = formatdate()
        string_to_sign = '\n'.join(('GET', '/keystone-test/', '', '', date))
        auth = 'AWS %s:%s' % (self.access_key, self.hmac_sha1(string_to_sign))
        response = self.client.get('/keystone-test/', HTTP_AUTHORIZATION=auth, HTTP_DATE=date)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_request_invalid_signature_returns_unauthorized(self):
        string_to_sign = '\n'.join(('GET', '/keystone-test/', '', '', formatdate()))
        auth = 'AWS %s:%s' % ('invalid accesskey', self.hmac_sha1(string_to_sign))
        response = self.client.get('/keystone-test/', HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)



# encoding: utf-8
from django.shortcuts import render
from django.core.urlresolvers import reverse
from ..api import router
from .utils import get_view_doc


def api_help(request):
    breadcrumblist = [
        ('API Help', '/api'),
    ]

    # iterate over all registered apis
    api_doc_list = []
    for prefix, viewset, basename in router.registry:
        api_doc = APIDoc(viewset=viewset, basename=basename)
        api_doc_list.append(api_doc)

    # order by name
    api_doc_list.sort()

    return render(request, 'api_help.html', {
        "breadcrumblist": breadcrumblist,
        "api_doc_list": api_doc_list,
    }, content_type="text/html")


class APIDoc(object):

    def __init__(self, viewset, basename):
        self.viewset = viewset
        self.basename = basename

    @property
    def name(self):
        return self.basename

    @property
    def view_name(self):
        list_name_prefix = router.routes[0].name
        view_name = list_name_prefix.format(basename=self.basename)
        return view_name

    @property
    def url(self):
        return reverse(self.view_name)

    @property
    def documentation(self):
        return get_view_doc(self.viewset)

    def __cmp__(self, other):
        if self.name == other.name:
            return 0
        elif self.name > other.name:
            return 1
        return -1
